//
// File:    test_ami_btree.cpp
// Author:  Octavian Procopiuc <tavi@cs.duke.edu>
// 
// Test file for class AMI_btree.
//

#include "app_config.h"
#include <portability.h>
#include <versions.h>

/***************************/
#include <sstream>
#include <fstream>
#include <fcntl.h>
#include <unistd.h>
#include <vector>
#include "mtimer.h"
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <dirent.h>
#include "segment.h"
#include "sdb.h"

/**************************/

VERSION(test_ami_btree_cpp, "$Id: test_ami_btree1.cpp,v 1.7 2009/03/18 04:28:17 lifeifei Exp $");

#include <cpu_timer.h>
#include <ami_btree.h>
#include <ami_stream.h>
#define SIZE_OF_STRUCTURE 256

// Key type.
//typedef TPIE_OS_OFFSET bkey_t;
typedef double bkey_t;// le

// Element type for the btree.
struct el_t {
  bkey_t key_;
  uint csize;
  uint cd;
  uint ppos;
  char data_[SIZE_OF_STRUCTURE-sizeof(bkey_t)-3*sizeof(uint)];
  
  el_t(bkey_t k, string d, uint n, uint distinct, uint pos): key_(k) 
  {
    csize=n;
    cd=distinct;
    strcpy(data_,d.data());
    ppos=pos;
  }
  el_t() {}
};

struct key_from_el {
  bkey_t operator()(const el_t& v) const { return v.key_; }
};
// Temporary distinction btw UN*X and WIN, since there are some
// problems with the MMAP collection implementation.
#ifdef _WIN32
typedef AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el,BTE_COLLECTION_UFS > u_btree_t;
#else
typedef AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el > u_btree_t;
#endif

typedef AMI_STREAM< el_t > stream_t;

// Template instantiations (to get meaningful output from gprof)
//template class AMI_btree_node<bkey_t,el_t,less<bkey_t>,key_from_el>;
//template class AMI_btree_leaf<bkey_t,el_t,less<bkey_t>,key_from_el>;
//template class AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el,BTE_COLLECTION_UFS >;
//template class AMI_STREAM< el_t >;


// This is 2**31-1, the max value returned by random().
#define MAX_RANDOM ((double)0x7fffffff)
// This is the max value that we want.
#define MAX_VALUE 1000000000
// The number of deletions to perform.
#define DELETE_COUNT 500

// Global variables.
TPIE_OS_OFFSET bulk_load_count;
TPIE_OS_OFFSET insert_count = 5000;
TPIE_OS_OFFSET range_search_lo = 0;
TPIE_OS_OFFSET range_search_hi = 10000000;


int get_treesize(const char * filename)
{
  char n[100]="";
  int size=0;
  struct dirent *w;
  struct stat buf;
  sprintf(n,"%s/btree.ent.l.blk",filename);
  if (stat(n, &buf) < 0) {
    fprintf(stderr, "Cannot stat %s\n", n);                                                                               
    exit(1);
  }
  size+=buf.st_size;
  sprintf(n,"%s/btree.ent.n.blk",filename);
  if (stat(n, &buf) < 0) {
    fprintf(stderr, "Cannot stat %s\n", n);                                                                               
    exit(1);
  }
  size+=buf.st_size;
  return size;
}

int get_cflsize(const char * filename)
{
  char dir[100];
  sprintf(dir,"%s/C",filename);
  DIR *d;
  long size=0;
  struct dirent *w;
  struct stat buf;
  char n[100]="";
  if (stat(dir, &buf) < 0) {
    return 0;
  }
  else
    {
      if(!S_ISDIR(buf.st_mode))
	{
	  if(S_ISLNK(buf.st_mode))
	    {
	      printf("%-ld\t%-s\n",0,dir);
	      exit(0);
	    }
	  else
	    {
	      printf("%-ld\t%-s\n",(long)(buf.st_blocks/2),dir);
	      exit(0);
	    }
	}//if
    }
  if ((d = opendir(dir)) == NULL) {
    printf("This is not supposed to happen.\n");
    exit(0);
  }
  
  while ((w = readdir(d)) != NULL) {
    sprintf(n, "%s/%s", dir, w->d_name);
      if ((strcmp(w->d_name, ".") == 0) ||
	  (strcmp(w->d_name, "..")==0)) continue;
      if (stat(n, &buf) < 0) {
	continue;
      }
      if ((strstr(w->d_name, "."))!=NULL) 
	  continue;
      size+=(long)buf.st_size;
      if (S_ISDIR(buf.st_mode)) {
	size+=get_cflsize(n);
      }
  }
    //stat(dir, &buf);
      //printf("%-ld\t%-s\n",size+(long)(buf.st_blocks/2),dir);
  closedir(d);
  return size;
}


int main(int argc, char *argv[]) {

  AMI_err ae;
  TPIE_OS_OFFSET i;
  TPIE_OS_OFFSET j;
  el_t s[DELETE_COUNT], ss;
  cpu_timer wt;
  unsigned long int treesize=0,cflsize=0;//,cflsize2=0;
  char *base_file = NULL;
  char * base_file_stream = NULL;
  if(argc!=5)
    {
      cerr<<"Usage:test_ami_btree <# of random samples> <trick thresh> <B size> datafile"<<endl<<flush;
      cerr<<argv[0]<<" "<<argv[1]<<" "<<argv[2]<<" "<<argv[3]<<" "<<argv[4]<<endl;
      exit(0);
    }
  int T=atoi(argv[2]);
  /*********build conflict list**************************/
  char data_folder[100]="";
  strcpy(data_folder,argv[4]);
  strcat(data_folder,"Index/");
  int Bsize=(int)((double)(atoi(argv[3])*1024)/(double)Tools::OBJSIZE);//modify this////////////////////////////////////////////////////////////
  char buf[100]="";
  stringstream conv, conv1;
  vector<u_btree_t *> ubt;
  char folder[100]="";
  bool flag=true;
  conv1<<argv[1];
  int k,slvl;
  int treeheight=0;
  MTimer t;
  conv1>>slvl;
  static int fanout;
  //cerr<<slvl<<" B+ trees..."<<endl;
  uint pagesize=atoi(argv[3])*1024;
     vector <segment> env;
     vector <unsigned long int> cn;
     vector <uint> cd;
     vector <uint> ppos;

  for(k=0;k<slvl;k++)
    {
      conv.str("");
      conv<<data_folder;
      conv<<"S"<<k;
      strcpy(folder,conv.str().data());
      conv<<"/"<<"env";
      //cerr<<"str:"<<conv.str().data()<<endl;
     
      //readsmpl r(conv.str().data());
      //read_data2vv(conv.str().data(), env,cn);  
      // Log debugging info from the application, but not from the library. 
      env.clear();
      cn.clear();
      cd.clear();
      ppos.clear();

      SDB::readEnvelop(conv.str().data(),env,cn, cd, ppos);
      t.go(); 
      tpie_log_init(TPIE_LOG_APP_DEBUG); 
      
      MM_manager.set_memory_limit(64*1024*1024);
      MM_manager.enforce_memory_limit();
      
      bulk_load_count=env.size();
      conv.str("");
      conv<<folder<<"/btree.ent"; 
      strcpy(buf,conv.str().data());
      base_file=const_cast <char*> (buf);      
      
      //////  Testing Bulk loading. ///////
      
      u_btree_t *u_btree;
      AMI_btree_params params;
      ///////////////////////////////////
      // params.node_block_factor = 1; //
      // params.leaf_block_factor = 1; //
      // params.leaf_cache_size = 32;  //
      // params.node_cache_size = 64;  //
      ///////////////////////////////////
     
      
      params.node_block_factor = (int)((double)(pagesize)/4048.0); 
      params.leaf_block_factor = (int)((double)(pagesize)/4048.0); 
      params.node_size_max = 0;
      params.leaf_size_max = 0;
      u_btree = (base_file == NULL) ? new u_btree_t(params): 
	new u_btree_t(base_file, AMI_WRITE_COLLECTION, params);
      
      if (!u_btree->is_valid()) {
	cerr << argv[0] << ": Error initializing btree. Aborting.\n";
	delete u_btree;
	exit(1);
      }
      fanout=u_btree->fetch_node(99999)->lk_capacity(pagesize);
      if (u_btree->size() == 0) {
	//	cerr<<"in"<<endl;
	conv.str("");
	conv<<folder<<"/btree.stream";
	
	base_file_stream=const_cast <char*> (conv.str().data());
	//cout << "\n";
	//	cout << "Element size: " << sizeof(el_t) << " bytes. "
	//   << "Key size: " << sizeof(bkey_t) << " bytes.\n";
	TPIE_OS_SRANDOM((unsigned int)TPIE_OS_TIME(NULL));
	
	// Timing stream write.
	//cout << "BEGIN Stream initialization\n";
	stream_t* is = (base_file_stream == NULL) ? new stream_t:
	  new stream_t(base_file_stream);
	//cout << "\tCreating stream with " << bulk_load_count << " conflict lists.\n";
	//wt.start();

	for (j = 0; j < bulk_load_count; j++) {
	  conv.str("");
	  conv<<folder<<"/C/C";
	  is->write_item(el_t(env[j].start.x, conv.str().data(), (uint)cn[j], cd[j], ppos[j]));
	  if(T>0)
	    cflsize+=((cn[j]<=(T*Bsize*pow(2.0,k)))?cn[j]:0);
	  else
	    cflsize+=cn[j];
	}
	//wt.stop();
	//	cout << "END Stream write " << wt << "\n";
	//wt.reset();
	//cout << "BEGIN Bulk Load\n";
	//    cout << "\tBulk loading from the stream created.\n";
	//cout << "\tSorting... " << flush;
	//wt.start();
	
	/* stream_t *os = new stream_t;
	if (u_btree->sort(is, os) != AMI_ERROR_NO_ERROR)
	  cerr << argv[0] << ": Error during sort.\n";
	else {
	  //wt.stop();
	  if ((ae = os->seek(0))!= AMI_ERROR_NO_ERROR) {
	    cout << "AMI_ERROR " << ae << " during interm.seek()" << endl;
	    exit(1);
	  }
	*/
	  //cout << "Done. " << wt << "\n";
	  //wt.reset();
	  //wt.start();
	  //cout << "\tLoading... " << flush;
	  if (u_btree->load_sorted(is,0.9,0.9) != AMI_ERROR_NO_ERROR)
	    cerr << argv[0] << ": Error during bulk loading.\n";
	  else
	    {
	      //cout << "Done. " << wt << "\n";
	    }
	  //wt.stop();
		
	//os->persist(PERSIST_DELETE);
	//delete os;
	///    wt.stop();
	// cout << "END Bulk Load "  << "\n";	
	delete is;
	treesize+=get_treesize(folder);
	//cflsize2+=get_cflsize(folder);// get this from smplobj.cpp
	flag=true;
      }
      ubt.push_back(u_btree);
      t.stop();
      t.update(); 
      
      treeheight+=u_btree->height();
      //      cout  /*<< u_btree->size() <<" elements. "*//* CFL size:" << get_cflsize(folder)/1024<<"KB."*/<<" Tree height: " 
      // << static_cast<TPIE_OS_OUTPUT_SIZE_T>(u_btree->height()) << ".\n";
      
      //wt.reset();
      sprintf(buf,"rm -f %s/btree.stream",folder);
      system(buf);
       
      
    }//for

  if(flag)
    {
      /*
	Total Trees size, Total CFL size, Total CFL+BTree size, 
	Total tree height, Avg tree height, Total build Btree atime,
	B+tree fanout level, pagesize
      */
      cout << t.elapsed<<" "<< treesize/1024<<" "; 
      cout << ((double)cflsize*Tools::OBJSIZE)/1024.0<<" "; 
      cout << (((double)cflsize*Tools::OBJSIZE)/1024.0+((double)treesize)/1024)<<" "; 
      cout <<treeheight<<" "<<(double)treeheight/slvl<<" "<<slvl;
      cout <<" "<<fanout<<" "<<pagesize/1024<<endl;
    }
  else
    {
      //////////////////a simple stabbing query///////////////////////////////////////
      bkey_t key=1999990.0;				 
      int top_k=1100;
      int num=0;
      //  //char data[SIZE_OF_STRUCTURE-sizeof(bkey_t)];	 //
      //  //memset(data,0,SIZE_OF_STRUCTURE-sizeof(bkey_t)); //
      fstream r;
      segment t;
      int m=0;
      while(num<top_k){
	if(m==ubt.size())
	  {
	    cerr<<"Found top:"<<num<<endl;
	    exit(0);
	  }
	u_btree_t *u_btree;
	u_btree=ubt[m++];
	
	if(u_btree->pred(key,ss))				 
	  {							 
	    cerr<<ss.key_<<" "<<ss.data_<<endl;		 
	  }	
	
	r.open(ss.data_,ios::binary|ios::in);
	vector <segment> vt;
	while(!r.eof())
	  {
	    r.read((char*)&t,sizeof(segment));
	    vt.push_back(t);
	  }
	r.close();
	num=vt.size();
	cerr<<"Return Conflict size:"<<num<<endl;
	//////////////////////////////////////////////////////////////////////////
	// tpie_stats_tree bts = u_btree->stats();		 		    //
	// cout << "Block collection statistics (global):\n"		    //
	// 	 << "\tGET BLOCK:    "						    //
	// 	 << AMI_COLLECTION::gstats().get(BLOCK_GET) << "\n"		    //
	// 	 << "\tPUT BLOCK:    "						    //
	// 	 << AMI_COLLECTION::gstats().get(BLOCK_PUT) << "\n"		    //
	// 	 << "\tNEW BLOCK     "						    //
	// 	 << AMI_COLLECTION::gstats().get(BLOCK_NEW) << "\n"		    //
	// 	 << "\tDELETE BLOCK: "						    //
	// 	 << AMI_COLLECTION::gstats().get(BLOCK_DELETE) << "\n"		    //
	//   ;								    //
	// cout << "Tree statistics:\n"					    //
	// 	 << "\tREAD (LEAF+NODE):    " 					    //
	// 	 << bts.get(LEAF_READ) + bts.get(NODE_READ) << "\n"		    //
	// 	 << "\tCREATE (LEAF+NODE):  " 					    //
	// 	 << bts.get(LEAF_CREATE) + bts.get(NODE_CREATE) << "\n"		    //
	// 	 << "\tFETCH (LEAF+NODE):   " 					    //
	// 	 << bts.get(LEAF_FETCH) + bts.get(NODE_FETCH) << "\n"		    //
	// 	 << "\tWRITE (LEAF+NODE):   " 					    //
	// 	 << bts.get(LEAF_WRITE) + bts.get(NODE_WRITE) << "\n"		    //
	// 	 << "\tDELETE (LEAF+NODE):  " 					    //
	// 	 << bts.get(LEAF_DELETE) + bts.get(NODE_DELETE) << "\n"		    //
	// 	 << "\tRELEASE (LEAF+NODE): " 					    //
	// 	 << bts.get(LEAF_RELEASE) + bts.get(NODE_RELEASE) << "\n"	    //
	//   ;								    //
	// cout << "Stream statistics (global):\n"				    //
	// 	 << "\tREAD ITEM:    "						    //
	// 	 << stream_t::gstats().get(ITEM_READ) << "\n"			    //
	// 	 << "\tWRITE ITEM:   "						    //
	// 	 << stream_t::gstats().get(ITEM_WRITE) << "\n"			    //
	// 	 << "\tSEEK ITEM:    "						    //
	// 	 << stream_t::gstats().get(ITEM_SEEK) << "\n"			    //
	// 	 << "\tREAD BLOCK:   "						    //
	// 	 << stream_t::gstats().get(BLOCK_READ) << "\n"			    //
	// 	 << "\tWRITE BLOCK:  "						    //
	// 	 << stream_t::gstats().get(BLOCK_WRITE) << "\n"			    //
	//   ;								    //
	//////////////////////////////////////////////////////////////////////////  
      }	
    }	//else		 
  vector <u_btree_t *>::iterator it;
  for(it=ubt.begin();it!=ubt.end();++it)
    {
      delete (*it);
    }
  system("rm -f *blk");
  system("rm -f .n.blk");
  
  return 0;
}
