//
// File:    test_ami_btree.cpp
// Author:  Octavian Procopiuc <tavi@cs.duke.edu>
// 
// Test file for class AMI_btree.
//

#include "app_config.h"
#include <portability.h>
#include <versions.h>

/***************************/
#include <sstream>
#include <fstream>
#include <fcntl.h>
#include <unistd.h>
#include <vector>
#include "mtimer.h"
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <dirent.h>
#include "segment.h"
#include "sdb.h"

/**************************/

VERSION(test_ami_btree3_cpp, "$Id: test_ami_btree3.cpp,v 1.6 2009/03/17 05:07:31 lifeifei Exp $");

#include <cpu_timer.h>
#include <ami_btree.h>
#include <ami_stream.h>
#define SIZE_OF_STRUCTURE 2048

// Key type.
//typedef TPIE_OS_OFFSET bkey_t;
typedef double bkey_t;// le

// Element type for the btree.
struct el_t {
  bkey_t key_;
  char data_[SIZE_OF_STRUCTURE-sizeof(bkey_t)];
  
  el_t(bkey_t k): key_(k) 
  {}
  el_t() {}
};

struct key_from_el {
  bkey_t operator()(const el_t& v) const { return v.key_; }
};
// Temporary distinction btw UN*X and WIN, since there are some
// problems with the MMAP collection implementation.
#ifdef _WIN32
typedef AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el,BTE_COLLECTION_UFS > u_btree_t;
#else
typedef AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el > u_btree_t;
#endif

typedef AMI_STREAM< el_t > stream_t;

// Template instantiations (to get meaningful output from gprof)
//template class AMI_btree_node<bkey_t,el_t,less<bkey_t>,key_from_el>;
//template class AMI_btree_leaf<bkey_t,el_t,less<bkey_t>,key_from_el>;
//template class AMI_btree< bkey_t,el_t,less<bkey_t>,key_from_el,BTE_COLLECTION_UFS >;
//template class AMI_STREAM< el_t >;


// This is 2**31-1, the max value returned by random().
#define MAX_RANDOM ((double)0x7fffffff)
// This is the max value that we want.
#define MAX_VALUE 1000000000
// The number of deletions to perform.
#define DELETE_COUNT 500

// Global variables.
TPIE_OS_OFFSET bulk_load_count;
TPIE_OS_OFFSET insert_count = 5000;
TPIE_OS_OFFSET range_search_lo = 0;
TPIE_OS_OFFSET range_search_hi = 10000000;


void prepareData(el_t& element, uint cfltsize,  uint distinct, uint sindex, uint eindex, vector<uint> level, vector<uint> index)
{	
	char* ptr=element.data_;
	int total=0;
	uint totalfile=eindex-sindex;
	memcpy(ptr, &cfltsize, sizeof(uint));
	ptr+=sizeof(uint);
	total+=sizeof(uint);

	memcpy(ptr, &distinct, sizeof(uint));
	ptr+=sizeof(uint);
	total+=sizeof(uint);

	memcpy(ptr, &totalfile, sizeof(uint));
	ptr+=sizeof(uint);
	total+=sizeof(uint);

	for (uint i=sindex;i<eindex;i++)
	{
		memcpy(ptr, &(level[i]), sizeof(uint));
		ptr+=sizeof(uint);
		total+=sizeof(uint);
		memcpy(ptr, &(index[i]), sizeof(uint));
		ptr+=sizeof(uint);
		total+=sizeof(uint);
	}
	if (total>SIZE_OF_STRUCTURE-sizeof(bkey_t))
	{
		cerr<<"data size too large, increase SIZE_OF_STRUCTUE"<<endl;
		exit(-1);
	}
} 

int get_treesize(const char * filename)
{
  char n[100]="";
  int size=0;
  struct dirent *w;
  struct stat buf;
  sprintf(n,"%s/btree.ent.l.blk",filename);
  if (stat(n, &buf) < 0) {
    fprintf(stderr, "Cannot stat %s\n", n);                                                                               
    exit(1);
  }
  size+=buf.st_size;
  sprintf(n,"%s/btree.ent.n.blk",filename);
  if (stat(n, &buf) < 0) {
    fprintf(stderr, "Cannot stat %s\n", n);                                                                               
    exit(1);
  }
  size+=buf.st_size;
  return size;
}

int get_cflsize(const char * filename)
{
  char dir[100];
  sprintf(dir,"%s/C",filename);
  DIR *d;
  long size=0;
  struct dirent *w;
  struct stat buf;
  char n[100]="";
  if (stat(dir, &buf) < 0) {
    return 0;
  }
  else
    {
      if(!S_ISDIR(buf.st_mode))
	{
	  if(S_ISLNK(buf.st_mode))
	    {
	      printf("%-ld\t%-s\n",0,dir);
	      exit(0);
	    }
	  else
	    {
	      printf("%-ld\t%-s\n",(long)(buf.st_blocks/2),dir);
	      exit(0);
	    }
	}//if
    }
  if ((d = opendir(dir)) == NULL) {
    printf("This is not supposed to happen.\n");
    exit(0);
  }
  
  while ((w = readdir(d)) != NULL) {
    sprintf(n, "%s/%s", dir, w->d_name);
      if ((strcmp(w->d_name, ".") == 0) ||
	  (strcmp(w->d_name, "..")==0)) continue;
      if (stat(n, &buf) < 0) {
	continue;
      }
      if ((strstr(w->d_name, "."))!=NULL) 
	  continue;
      size+=(long)buf.st_size;
      if (S_ISDIR(buf.st_mode)) {
	size+=get_cflsize(n);
      }
  }
    //stat(dir, &buf);
      //printf("%-ld\t%-s\n",size+(long)(buf.st_blocks/2),dir);
  closedir(d);
  return size;
}


int main(int argc, char *argv[]) {

  AMI_err ae;
  TPIE_OS_OFFSET i;
  TPIE_OS_OFFSET j;
  el_t s[DELETE_COUNT], ss;
  cpu_timer wt;
  unsigned long int treesize=0,cflsize=0;//,cflsize2=0;
  char *base_file = NULL;
  char * base_file_stream = NULL;
  if(argc!=4)
    {
      cerr<<"Usage:test_ami_btree <# of random samples> <B size> datafile"<<endl<<flush;
      cerr<<argv[0]<<" "<<argv[1]<<" "<<argv[2]<<" "<<argv[3]<<endl;
      exit(0);
    }

  /*********build conflict list**************************/
  char data_folder[100]="";
  strcpy(data_folder,argv[3]);
  strcat(data_folder,"Index/");
  int Bsize=atoi(argv[2])*1024/40;//modify this////////////////////////////////////////////////////////////
  char buf[100]="";
  stringstream conv, conv1;
  vector<u_btree_t *> ubt;
  char folder[100]="";
  bool flag=true;
  conv1<<argv[1];
  int k,slvl;
  int treeheight=0;
  MTimer t;
  conv1>>slvl;
  static int fanout;
  //cerr<<slvl<<" B+ trees..."<<endl;
  uint pagesize=atoi(argv[2])*1024;
  vector <segment> env;
  vector <uint> cn;
  vector <uint> cd;
  vector <uint> cdepend;
  vector <uint> clevel;
  vector <uint> cindex;
  tpie_log_init(TPIE_LOG_APP_DEBUG); 
  
  MM_manager.ignore_memory_limit();
  for(k=0;k<slvl;k++)
    {
      conv.str("");
      conv<<data_folder;
      conv<<"S"<<k;
      strcpy(folder,conv.str().data());
      conv<<"/"<<"env";
      //cerr<<"str:"<<conv.str().data()<<endl;
     
      //readsmpl r(conv.str().data());
      // Log debugging info from the application, but not from the library. 
	env.clear();
	cn.clear();
	cd.clear();
	cdepend.clear();
	clevel.clear();
	cindex.clear();
	SDB::readREnvelop(conv.str().data(), env, cn,cd,  cdepend,clevel, cindex);
      t.go(); 

      
      bulk_load_count=env.size();
      conv.str("");
      conv<<folder<<"/btree.ent"; 
      strcpy(buf,conv.str().data());
      base_file=const_cast <char*> (buf);      
      
      //////  Testing Bulk loading. ///////
      
      u_btree_t *u_btree;
      AMI_btree_params params;
      ///////////////////////////////////
      // params.node_block_factor = 1; //
      // params.leaf_block_factor = 1; //
      // params.leaf_cache_size = 32;  //
      // params.node_cache_size = 64;  //
      ///////////////////////////////////
     
      
      params.node_block_factor = (int)((double)(pagesize)/4048.0); 
      params.leaf_block_factor = (int)((double)(pagesize)/4048.0); 
      params.node_size_max = 0;
      params.leaf_size_max = 0;
      u_btree = (base_file == NULL) ? new u_btree_t(params): 
	new u_btree_t(base_file, AMI_WRITE_COLLECTION, params);
      
      if (!u_btree->is_valid()) {
	cerr << argv[0] << ": Error initializing btree. Aborting.\n";
	delete u_btree;
	exit(1);
      }
      fanout=u_btree->fetch_node(99999)->lk_capacity(pagesize);
	//	cerr<<"in"<<endl;
	conv.str("");
	conv<<folder<<"/btree.stream";
	
	base_file_stream=const_cast <char*> (conv.str().data());
	//cout << "\n";
	//	cout << "Element size: " << sizeof(el_t) << " bytes. "
	//   << "Key size: " << sizeof(bkey_t) << " bytes.\n";
	TPIE_OS_SRANDOM((unsigned int)TPIE_OS_TIME(NULL));
	
	// Timing stream write.
	//cout << "BEGIN Stream initialization\n";
	stream_t* is = (base_file_stream == NULL) ? new stream_t:
	  new stream_t(base_file_stream);
	//cout << "\tCreating stream with " << bulk_load_count << " conflict lists.\n";
	//wt.start();
	uint sindex=0, eindex=0;
	for (j = 0; j < bulk_load_count; j++) {
	  el_t element(env[j].start.x);
	  eindex=sindex+cdepend[j];
	  prepareData(element, cn[j], cd[j], sindex, eindex, clevel, cindex); 
	  sindex+=cdepend[j];
	  is->write_item(element);
	  //	  cout<<"Element:"<<env[j].start.x<<" "<<conv.str().data()<<" "<<cn[j]<<endl;
	  //delete(cflpath[j]);
	  /*	  if(T>0)
	    cflsize+=((cn[j]<=(T*Bsize*pow(2.0,k)))?cn[j]:0);
	  else
	  cflsize+=cn[j];*/
	}

	/*stream_t *os = new stream_t;
	if (u_btree->sort(is, os) != AMI_ERROR_NO_ERROR)
	  cerr << argv[0] << ": Error during sort.\n";
	else {
	  //wt.stop();
	  if ((ae = os->seek(0))!= AMI_ERROR_NO_ERROR) {
	    cout << "AMI_ERROR " << ae << " during interm.seek()" << endl;
	    exit(1);
	  }*/
	
	  //cout << "Done. " << wt << "\n";
	  //wt.reset();
	  //wt.start();
	  //cout << "\tLoading... " << flush;
	  if (u_btree->load_sorted(is,0.9,0.9) != AMI_ERROR_NO_ERROR)
	    cerr << argv[0] << ": Error during bulk loading.\n";
	//os->persist(PERSIST_DELETE);
	//delete os;
	///    wt.stop();
	// cout << "END Bulk Load "  << "\n";	
	delete is;
	treesize+=get_treesize(folder);
	cflsize+=get_cflsize(folder);
	
	flag=true;
      
      ubt.push_back(u_btree);
      t.stop();
      t.update(); 
      
      treeheight+=u_btree->height();
      //      cout  /*<< u_btree->size() <<" elements. "*//* CFL size:" << get_cflsize(folder)/1024<<"KB."*/<<" Tree height: " 
      // << static_cast<TPIE_OS_OUTPUT_SIZE_T>(u_btree->height()) << ".\n";
      
      //wt.reset();
      sprintf(buf,"rm -f %s/btree.stream",folder);
      system(buf);
       
      
    }//for

  if(flag)
    {
      /*
	Total Trees size, Total CFL size, Total CFL+BTree size, 
	Total tree height, Avg tree height, Total build Btree atime,
	B+tree fanout level, pagesize
      */
      
      cout << t.elapsed<<" "<< treesize/1024<<" "; 
      cout << (double)cflsize/1024.0<<" "; 
      cout << ((double)cflsize/1024.0)+((double)treesize/1024)<<" "; 
      cout <<treeheight<<" "<<(double)treeheight/slvl<<" "<<slvl;
      cout <<" "<<fanout<<" "<<pagesize/1024<<endl;
    }
  vector <u_btree_t *>::iterator it;
  for(it=ubt.begin();it!=ubt.end();++it)
    {
      delete (*it);
    }
  system("rm -f *blk");
  system("rm -f .n.blk");
  
  return 0;
}
